from typing import Union, List

try:
    from typing import Literal
except ImportError:
    from typing_extensions import Literal

from .base import BaseFunction
from ..properties.constant import Constant
from ..structures.order import Order
from ..properties.property import Property
from ..structures.cast_operator import CastOperator
from ..structures.case import Case


class FirstValue(BaseFunction):
    obj: Literal['functions.FirstValue'] = 'functions.FirstValue'
    property: Union[Property, BaseFunction, Constant, CastOperator, Case]
    order: List[Order]
    partition: List[Union[Property, BaseFunction, Constant, CastOperator, Case]] = []
    alias: str = None

    def to_sql(self) -> str:
        sql = ""

        if self.partition:
            sql = f"PARTITION BY {', '.join(map(str, self.partition))}"

        sql = f"{sql} ORDER BY {', '.join(map(str, self.order))}"

        sql = f"FIRST_VALUE({self.property}) OVER ({sql})"

        if self.alias is not None:
            sql = f"{sql} AS {self.alias}"

        return sql



from typing import Union, Optional, List

try:
    from typing import Literal
except ImportError:
    from typing_extensions import Literal

from ..structures.distinct import Distinct
from ..functions.base import BaseFunction
from ..properties import Array
from ..properties.property import Property
from ..structures.cast_operator import CastOperator
from ..structures.case import Case
from ..structures.order import Order


class ArrayAgg(BaseFunction):
    obj: Literal['functions.StringAgg'] = 'functions.ArrayAgg'
    property: Union[Property, BaseFunction, Array, CastOperator, Distinct, Case]
    order: Optional[List[Order]]
    position: Optional[int]
    alias: str = None

    def to_sql(self) -> str:
        sql = f"{self.property}"

        if self.order:
            sql = f"{sql} ORDER BY {', '.join(map(str, self.order))}"

        sql = f"ARRAY_AGG({sql})"

        if self.position:
            sql = f"({sql})[{self.position}]"

        if self.alias is not None:
            sql = f"{sql} AS {self.alias}"

        return sql



from typing import List, Union, Optional

from pydantic import BaseModel

try:
    from typing import Literal
except ImportError:
    from typing_extensions import Literal

from ..base import BaseType
from ..properties.property import Property
from ..properties.constant import Constant
from ..structures.comparision import Comparision
from ..structures.operator import Operator
from ..functions.base import BaseFunction


class Case(BaseType):
    # TODO: Docs!
    class Condition(BaseModel):
        when: Comparision
        then: Union[BaseFunction, Property, Constant, Operator]

        def __str__(self):
            return f"WHEN {self.when} THEN {self.then}"

    obj: Literal['structures.Case'] = 'structures.Case'
    conditions: List[Condition]
    alternative: Optional[Union[BaseFunction, Property, Constant]] = None
    alias: str = None

    def to_sql(self) -> str:
        sql = "CASE " + " ".join(map(str, self.conditions))
        if self.alternative:
            sql = f"{sql} ELSE {self.alternative}"

        sql = f"({sql} END)"

        if self.alias is not None:
            sql = f"{sql} AS {self.alias}"

        return sql
